import java.util.Properties;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.ejb.ActivationConfigProperty;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.MessageProducer;
import javax.jms.Session;
import org.apache.log4j.Logger;
import org.jboss.ejb3.annotation.ResourceAdapter;

/**
 * TODO class description
 * 
 * @author Shan Narayanan
 */
@MessageDriven(name = "BackendMDB", activationConfig = {
		//@ActivationConfigProperty(propertyName = "maxSession", propertyValue = "30"),
		@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Topic"),
		@ActivationConfigProperty(propertyName = "destination", propertyValue = "topic/AlchemyJobTopic"),
		@ActivationConfigProperty(propertyName = "subscriptionDurability", propertyValue = "Durable"),
		//@ActivationConfigProperty(propertyName = "subscriptionName", propertyValue="BackendMDB"),
		@ActivationConfigProperty(propertyName = "clientID", propertyValue="Alchemy"),
		@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge"),
		@ActivationConfigProperty(propertyName = "messageSelector", propertyValue = "MESSAGE_TYPE<>'SMSOptOut' AND MESSAGE_TYPE <>'BatchTrigger'")
		})
public class BackendMDB implements MessageListener
{
	private final Logger log = Logger.getLogger(getClass());
	
	// Account Operations
	public static final String CREATE_ACCOUNT = "CreateAccount";
	public static final String UPDATE_ACCOUNT = "UpdateAccount";
	public static final String DELETE_ACCOUNT = "DeleteAccount";
	public static final String TRANSFER_ACCOUNT = "TransferAccount";
	
	// Pooling Operations
	public static final String ADD_ACCOUNT_TO_GROUP = "AddAccountToGroup";
	public static final String REMOVE_ACCOUNT_FROM_GROUP = "RemoveAccountFromGroup";
	public static final String CREATE_ACCOUNT_GROUP = "CreateAccountGroup";
	public static final String DELETE_ACCOUNT_GROUP = "DeleteAccountGroup";

	// Discount Operations
	public static final String CREATE_DISCOUNT = "CreateDiscount";
	public static final String UPDATE_DISCOUNT = "UpdateDiscount";
	public static final String DELETE_DISCOUNT = "DeleteDiscount";
	public static final String CREATE_SEGMENT = "CreateSegment";
	public static final String UPDATE_SEGMENT = "UpdateSegment";
	public static final String DELETE_SEGMENT = "DeleteSegment";
	public static final String ADD_LOCATION_SEGMENT = "AddLocationSegment";
	public static final String REMOVE_LOCATION_SEGMENT = "RemoveLocationSegment";
	public static final String MERGE_SEGMENT = "MergeSegment";
	
	// Reward Operations
	public static final String CREATE_AWARD = "CreateAward";
	public static final String CREATE_ADJUSTMENT = "CreateAdjustment";
	public static final String CREATE_REDEMPTION = "CreateRedemption";
	public static final String CREATE_POOLED_REDEMPTION = "CreatePooledRedemption";
	public static final String UPDATE_REDEMPTION = "UpdateRedemption";
	public static final String REVERSE_REDEMPTION = "ReverseRedemption";
	public static final String CHARGE_ACCOUNT = "ChargeAccount";
	public static final String PURGE_REWARD = "PurgeReward";
	public static final String HOLD_RELEASE_REWARD = "HoldReleaseReward";
	public static final String EXPIRE_REWARD = "ExpireReward";
	
	// Transaction Operations
	public static final String CREATE_TRANSACTION = "CreateTransaction";
	public static final String UPDATE_TRANSACTION = "UpdateTransaction";
	public static final String CREATE_REWARD = "CreateReward";
	
	// Retail Operations
	public static final String FINALIZE_TRANSACTION = "FinalizeTransaction";
	
	//Triggers
	public static final String EMAIL_TRIGGER = "EmailTrigger";
	public static final String SMS_TRIGGER = "SmsTrigger";
	
	@Resource(name = "centralQueueURL")
	public String centralQueueURL;
	
	@Resource
  private MessageDrivenContext mdc;
	
	private Context jndiContext;
	private ConnectionFactory connectionFactory;
	private Connection connection;
	private Session session;
	private Destination destination;
	private MessageProducer producer;
	private String destinationName;

	/**
	 * 
	 */
	public BackendMDB()
	{
	}
	
	@PostConstruct
	public void init()
	{
		if (log.isDebugEnabled())
			log.debug("Message received");
		destinationName = "dynamicTopics/Alchemy.Central";

		/*
		 * Create a JNDI API InitialContext object
		 */
		try
		{
			Properties props = new Properties();
			props.setProperty(Context.INITIAL_CONTEXT_FACTORY,
					"org.apache.activemq.jndi.ActiveMQInitialContextFactory");
			props.setProperty(Context.PROVIDER_URL, centralQueueURL);
			jndiContext = new InitialContext(props);
		}
		catch (NamingException e)
		{
			log.error("Could not create JNDI API context: " , e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
	 */
	public void onMessage(Message msg)
	{
		if (log.isDebugEnabled())
			log.debug("Message received");
		
		try
		{
			/*
			 * Look up connection factory and destination.
			 */
			connectionFactory = (ConnectionFactory) jndiContext
					.lookup("ConnectionFactory");
			destination = (Destination) jndiContext.lookup(destinationName);

			/*
			 * Create connection. Create session from connection; false means session is
			 * not transacted. Create sender and text message. Send messages, varying
			 * text slightly. Send end-of-messages message. Finally, close connection.
			 */
			connection = connectionFactory.createConnection();
			session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
			producer = session.createProducer(destination);
			
			MapMessage mm = (MapMessage) msg;
			log.debug("Entered BackendMDB Processing: "+mm.getJMSMessageID());
			if (session == null)
			{
				mdc.setRollbackOnly();
				return;
			}
			
			MapMessage nm = session.createMapMessage();
			
			// Account Events
			if (mm.itemExists(CREATE_ACCOUNT))
			{
				nm.setString(CREATE_ACCOUNT, mm.getString(CREATE_ACCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_ACCOUNT));
			}
			else if (mm.itemExists(UPDATE_ACCOUNT))
			{
				nm.setString(UPDATE_ACCOUNT, mm.getString(UPDATE_ACCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(UPDATE_ACCOUNT));
			}
			else if (mm.itemExists(DELETE_ACCOUNT))
			{
				nm.setString(DELETE_ACCOUNT, mm.getString(DELETE_ACCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(DELETE_ACCOUNT));
			}
			else if (mm.itemExists(TRANSFER_ACCOUNT))
			{
				nm.setString(TRANSFER_ACCOUNT, mm.getString(TRANSFER_ACCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(TRANSFER_ACCOUNT));
			}
			
			// Pooling Events
			if (mm.itemExists(ADD_ACCOUNT_TO_GROUP))
			{
				nm.setString(ADD_ACCOUNT_TO_GROUP, mm.getString(ADD_ACCOUNT_TO_GROUP));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(ADD_ACCOUNT_TO_GROUP));
			}
			else if (mm.itemExists(REMOVE_ACCOUNT_FROM_GROUP))
			{
				nm.setString(REMOVE_ACCOUNT_FROM_GROUP, mm.getString(REMOVE_ACCOUNT_FROM_GROUP));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(REMOVE_ACCOUNT_FROM_GROUP));
			}
			else if (mm.itemExists(CREATE_ACCOUNT_GROUP))
			{
				nm.setString(CREATE_ACCOUNT_GROUP, mm.getString(CREATE_ACCOUNT_GROUP));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_ACCOUNT_GROUP));
			}
			else if (mm.itemExists(DELETE_ACCOUNT_GROUP))
			{
				nm.setString(DELETE_ACCOUNT_GROUP, mm.getString(DELETE_ACCOUNT_GROUP));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(DELETE_ACCOUNT_GROUP));
			}
			
			// Discount Events
			if (mm.itemExists(CREATE_DISCOUNT))
			{
				nm.setString(CREATE_DISCOUNT, mm.getString(CREATE_DISCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_DISCOUNT));
			}
			else if (mm.itemExists(UPDATE_DISCOUNT))
			{
				nm.setString(UPDATE_DISCOUNT, mm.getString(UPDATE_DISCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(UPDATE_DISCOUNT));
			}
			else if (mm.itemExists(DELETE_DISCOUNT))
			{
				nm.setString(DELETE_DISCOUNT, mm.getString(DELETE_DISCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(DELETE_DISCOUNT));
			}
			
			// Segment Events
			if (mm.itemExists(CREATE_SEGMENT))
			{
				nm.setString(CREATE_SEGMENT, mm.getString(CREATE_SEGMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_SEGMENT));
			}
			else if (mm.itemExists(UPDATE_SEGMENT))
			{
				nm.setString(UPDATE_SEGMENT, mm.getString(UPDATE_SEGMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(UPDATE_SEGMENT));
			}
			else if (mm.itemExists(DELETE_SEGMENT))
			{
				nm.setString(DELETE_SEGMENT, mm.getString(DELETE_SEGMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(DELETE_SEGMENT));
			}
			else if (mm.itemExists(ADD_LOCATION_SEGMENT))
			{
				nm.setString(ADD_LOCATION_SEGMENT, mm.getString(ADD_LOCATION_SEGMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(ADD_LOCATION_SEGMENT));
			}
			else if (mm.itemExists(REMOVE_LOCATION_SEGMENT))
			{
				nm.setString(REMOVE_LOCATION_SEGMENT, mm.getString(REMOVE_LOCATION_SEGMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(REMOVE_LOCATION_SEGMENT));
			}
			else if (mm.itemExists(MERGE_SEGMENT))
			{
				nm.setString(MERGE_SEGMENT, mm.getString(MERGE_SEGMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(MERGE_SEGMENT));
			}
			
			// Reward Events
			if (mm.itemExists(CREATE_AWARD))
			{
				nm.setString(CREATE_AWARD, mm.getString(CREATE_AWARD));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_AWARD));
			}
			else if (mm.itemExists(CREATE_ADJUSTMENT))
			{
				nm.setString(CREATE_ADJUSTMENT, mm.getString(CREATE_ADJUSTMENT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_ADJUSTMENT));
			}
			else if (mm.itemExists(CREATE_REDEMPTION))
			{
				nm.setString(CREATE_REDEMPTION, mm.getString(CREATE_REDEMPTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_REDEMPTION));
			}
			else if (mm.itemExists(CREATE_POOLED_REDEMPTION))
			{
				nm.setString(CREATE_POOLED_REDEMPTION, mm.getString(CREATE_POOLED_REDEMPTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_POOLED_REDEMPTION));
			}
			else if (mm.itemExists(UPDATE_REDEMPTION))
			{
				nm.setString(UPDATE_REDEMPTION, mm.getString(UPDATE_REDEMPTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(UPDATE_REDEMPTION));
			}
			else if (mm.itemExists(REVERSE_REDEMPTION))
			{
				nm.setString(REVERSE_REDEMPTION, mm.getString(REVERSE_REDEMPTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(REVERSE_REDEMPTION));
			}
			else if (mm.itemExists(CHARGE_ACCOUNT))
			{
				nm.setString(CHARGE_ACCOUNT, mm.getString(CHARGE_ACCOUNT));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CHARGE_ACCOUNT));
			}
			else if (mm.itemExists(PURGE_REWARD))
			{
				nm.setString(PURGE_REWARD, mm.getString(PURGE_REWARD));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(PURGE_REWARD));
			}
			else if (mm.itemExists(HOLD_RELEASE_REWARD))
			{
				nm.setString(HOLD_RELEASE_REWARD, mm.getString(HOLD_RELEASE_REWARD));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(HOLD_RELEASE_REWARD));
			}
			
			// Transaction Events
			if (mm.itemExists(CREATE_TRANSACTION))
			{
				nm.setString(CREATE_TRANSACTION, mm.getString(CREATE_TRANSACTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_TRANSACTION));
			}
			else if (mm.itemExists(UPDATE_TRANSACTION))
			{
				nm.setString(UPDATE_TRANSACTION, mm.getString(UPDATE_TRANSACTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(UPDATE_TRANSACTION));
			}
			else if (mm.itemExists(CREATE_REWARD))
			{
				nm.setString(CREATE_REWARD, mm.getString(CREATE_REWARD));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(CREATE_REWARD));
			}
			else if (mm.itemExists(FINALIZE_TRANSACTION))
			{
				nm.setString(FINALIZE_TRANSACTION, mm.getString(FINALIZE_TRANSACTION));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(FINALIZE_TRANSACTION));
			}
			else if (mm.itemExists(EXPIRE_REWARD))
			{
				nm.setString(EXPIRE_REWARD, mm.getString(EXPIRE_REWARD));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(EXPIRE_REWARD));
			}
			
			//Trigger Events
			if (mm.itemExists(EMAIL_TRIGGER))
			{
				nm.setString(EMAIL_TRIGGER, mm.getString(EMAIL_TRIGGER));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(EMAIL_TRIGGER));
			}
			else if (mm.itemExists(SMS_TRIGGER))
			{
				nm.setString(SMS_TRIGGER, mm.getString(SMS_TRIGGER));
				if (log.isTraceEnabled())
					log.trace("Message : " + mm.getString(SMS_TRIGGER));
			}
			
			producer.send(nm);
			session.close();
			connection.close();
			log.debug("Exited BackendMDB Processing: "+mm.getJMSMessageID());
		}
		catch (NamingException e)
		{
			log.error("JNDI API lookup failed: " , e);
			mdc.setRollbackOnly();
		}
		catch (JMSException e)
		{
			log.error("An error occurred : " , e);
			mdc.setRollbackOnly();
		}
		finally
		{
			try
			{
				if (connection != null)
					connection.close();
			}
			catch (JMSException e)
			{
				log.error("Error closing the connection : " , e);
			}
		}
	}
}